@@ -117,10 +117,14 @@ checkNewPackageVersionNumber <- function(pkgdir)
 {
     dcf <- read.dcf(file.path(pkgdir, "DESCRIPTION"))
     version <- dcf[, "Version"]
-        if(!grepl("^0[-.]99[-.][0-9]+$", version))
+        if (!grepl("^0+[-.][0-9]+[-.][0-9]+$", version))
+            handleWarning(
+                "New package x version starting with non-zero value ",
+                "(e.g., 1.y.z, 2.y.z); got ", sQuote(version), ".")
+        if(!grepl("^[0-9]+[-.]99[-.][0-9]+$", version))
             handleError(
-                "New package version starting with 0.99.* (e.g., 0.99.0, ",
-                "0.99.1, ...); got ",sQuote(version), ".")
+                "New package y version not 99 (e.g., x.99.z, ",
+                "x.99.z, ...); got ",sQuote(version), ".")
 }
 
 checkVersionNumber <- function(pkgdir)
@@ -326,9 +330,36 @@ checkBiocViews <- function(pkgdir)
     return(dirty)
 }
 
+.checkDescription <- function(desc) {
+    dcf <- read.dcf(desc)
+    if ("Description" %in% colnames(dcf)) {
+        desc_field <- dcf[, "Description"]
+        desc_words <- lengths(strsplit(desc_field, split = "[[:space:]]+"))
+        desc_sentences <- length(gregexpr("[[:alnum:] ][.!?]", desc_field)[[1]])
+        msg <- "The Description field in the DESCRIPTION is made up by less
+            than 3 sentences. Please consider expanding this field, and
+            structure it as a full paragraph"
+
+        if (nchar(desc_field) < 50 || desc_words < 20) # values chosen sensibly in a data-driven manner
+            handleWarning(
+                "Description field in the DESCRIPTION file is too concise"
+            )
+        else if (desc_sentences < 3)
+            handleNote(paste(strwrap(msg), collapse="\n"))
+    }
+}
+
+.checkORCID <- function(orcid) 
+{
+    re <- "^[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{3}[0-9X]$"
+    grepl(re, orcid)
+}
+
+
 checkBBScompatibility <- function(pkgdir, source_tarball)
 {
     lines <- readLines(file.path(pkgdir, "DESCRIPTION"), warn=FALSE)
+    desc <- file.path(pkgdir, "DESCRIPTION")
     handleCheck("Checking for blank lines in DESCRIPTION...")
     if (any(nchar(lines)==0))
     {
@@ -337,13 +368,16 @@ checkBBScompatibility <- function(pkgdir, source_tarball)
     }
     handleCheck("Checking if DESCRIPTION is well formatted...")
     dcf <- tryCatch({
-        read.dcf(file.path(pkgdir, "DESCRIPTION"))
+        read.dcf(desc)
     }, error = function(err) {
         handleError("DESCRIPTION is malformed.")
         handleMessage(conditionMessage(err))
         return()
     })
 
+    handleCheck("Checking for proper Description: field...")
+    .checkDescription(desc)
+
     handleCheck("Checking for whitespace in DESCRIPTION field names...")
     if (any(grepl("\\s", colnames(dcf))))
     {
@@ -368,9 +402,12 @@ checkBBScompatibility <- function(pkgdir, source_tarball)
     }
     handleCheck("Checking for valid maintainer...")
     if (!source_tarball){
-        if (("Authors@R" %in% colnames(dcf)) &
-            any((c("Author","Maintainer") %in% colnames(dcf))))
-            handleWarning("Use Authors@R (preferred) or Author/Maintainer fields not both.")
+        if (("Authors@R" %in% colnames(dcf)) & any((c("Author","Maintainer") %in% colnames(dcf)))){
+            handleError("Use Authors@R field not Author/Maintainer fields. Do not use both.")
+        } else {
+            if (any((c("Author","Maintainer") %in% colnames(dcf))))
+                handleError("Do not use Author/Maintainer fields. Use Authors@R.")
+        }
     }
 
     maintainer <- NULL
@@ -390,8 +427,22 @@ checkBBScompatibility <- function(pkgdir, source_tarball)
             handleError("Authors@R must evaluate to 'person' object.")
             return()
         }
+        fnd <- vapply(people, FUN.VALUE=logical(1), USE.NAMES=FALSE,
+                      FUN=function(person){ "cre" %in% person$role})
+        if (length(which(fnd)) > 1L){
+            handleError("Designated only one maintainer with Authors@R [cre].")
+        }
         for (person in people)
         {
+            if ("ORCID" %in% names(person$comment)) {
+                orcid <- person$comment[["ORCID"]]
+                validID <- .checkORCID(orcid)
+                if (!validID)
+                    handleNote(
+                        "Invalid ORCID ID for ",
+                        person$given, " ", person$family
+                    )
+            }
             if ("cre" %in% person$role)
             {
                 email <- person$email
@@ -417,16 +468,17 @@ checkBBScompatibility <- function(pkgdir, source_tarball)
             return()
         }
     } else if ("Maintainer" %in% colnames(dcf)) {
-        maintainer <- dcf[,"Maintainer"]
+        handleError("Remove Maintainer field. Use Authors@R [cre] designation.")
+        return()
     } else {
-        handleError("No Maintainer or Authors@R [cre] field in DESCRIPTION file.")
+        handleError("No Authors@R [cre] field in DESCRIPTION file.")
         return()
     }
     # now need to make sure that regexes work, a la python/BBS
+    # I think R CMD check now does this already but can't hurt to keep
     regex = '(.*\\S)\\s*<(.*)>\\s*'
     match <- regexec(regex, maintainer)[[1]]
     match.length <- attr(match, "match.length")
-    #if (!  (all(match)  > 0) && (all(match.length) > 0) )
     if (all(match == -1) && all(match.length == -1))
     {
         handleError("Maintainer field in DESCRIPTION file is malformed.")
@@ -552,12 +604,6 @@ checkVigDirExists <- function(pkgdir, vigdir)
 {
     if (!file.exists(vigdir))
     {
-        if (isInfrastructurePackage(pkgdir))
-        {
-            .msg("  Infrastructure package, vignette not required.",
-                indent=2)
-            return(FALSE)
-        }
         handleError("No 'vignettes' directory.")
         return(FALSE)
     } else {
@@ -901,6 +947,15 @@ checkCodingPractice <- function(pkgdir, parsedCode, package_name)
             handleMessage(msg, indent=8)
     }
 
+    # pkg:fun...
+    msg_sc <- checkSingleColon(Rdir)
+    if (length(msg_sc)) {
+        handleError(" Use double colon for qualified imports: 'pkg::foo()'")
+        handleMessage("Found in files:", indent=6)
+        for (msg in msg_sc)
+            handleMessage(msg, indent=8)
+    }
+
     # T/F
     res <- checkLogicalUseFiles(pkgdir)
     pkgname <- basename(pkgdir)
@@ -998,6 +1053,30 @@ check1toN <- function(Rdir){
     msg_seq <- unlist(msg_seq)
 }
 
+checkSingleColon <- function(Rdir, avail_pkgs = character(0L)) {
+
+    rfiles <- dir(Rdir, pattern = "\\.[Rr]$", full.names = TRUE)
+    names(rfiles) <- basename(rfiles)
+    colon_pres <- lapply(rfiles, function(rfile) {
+        tokens <- getParseData(parse(rfile, keep.source = TRUE))
+        tokens <- tokens[tokens[,"token"] != "expr", ,drop=FALSE]
+        colons <- which(tokens[,"text"] == ":") - 1
+        colons <- colons[grepl("[[:alpha:]]", tokens[colons, "text"])]
+        tokens[colons, , drop = FALSE]
+    })
+    colon_pres <- Filter(nrow, colon_pres)
+    if (length(colon_pres))
+        avail_pkgs <- BiocManager::available()
+    msg_sc <- lapply(names(colon_pres), function(rfile, framelist) {
+        tokens <- framelist[[rfile]]
+        tokens <- tokens[tokens[, "text"] %in% avail_pkgs, , drop = FALSE]
+        sprintf(
+            "%s (line %d, column %d)",
+            rfile, tokens[, "line1"], tokens[, "col1"]
+        )
+    }, framelist = colon_pres)
+    msg_sc <- unlist(msg_sc)
+}
 
 checkClassEqUsage <- function(pkgdir){
 
@@ -1226,6 +1305,7 @@ checkUsageOfDont <- function(pkgdir)
         pattern="\\.Rd$", ignore.case=TRUE, full.names=TRUE)
 
     hasBad <- rep(FALSE, length(manpages))
+    hasdontrun <- rep(FALSE, length(manpages))
     for (dx in seq_along(manpages))
     {
         manpage <- manpages[dx]
@@ -1242,8 +1322,20 @@ checkUsageOfDont <- function(pkgdir)
             dontrunVec <- vapply(exampleCode, grepl, logical(1),
                                   pattern="\\\\dontrun", perl=TRUE,
                                   USE.NAMES=FALSE)
-            if (any(donttestVec | dontrunVec))
+            ## check for the 'internal' keyword - this will be a false positive
+            keyword <- unlist(lapply(rd,
+                function(x) attr(x, "Rd_tag") == "\\keyword"))
+            if (any(keyword)) {
+                internalVec <- vapply(as.character(rd[keyword]), grepl, logical(1),
+                                     pattern="internal", USE.NAMES=FALSE)
+            } else {
+                internalVec <- FALSE
+            }
+            if (any(donttestVec | dontrunVec) & !any(internalVec))
                 hasBad[dx] = TRUE
+
+            if (any(dontrunVec) & !any(internalVec))
+                hasdontrun[dx] = TRUE
         }
     }
     if (any(hasBad)){
@@ -1255,6 +1347,14 @@ checkUsageOfDont <- function(pkgdir)
             handleMessage(f, indent=8)
         }
     }
+    if (any(hasdontrun)){
+        handleNote("Use donttest{} instead of dontrun{}.")
+        handleMessage("Found in the following files:", indent=6)
+        for(f in basename(manpages)[hasdontrun]){
+            handleMessage(f, indent=8)
+        }
+     }
+
 }
 
 checkNEWS <- function(pkgdir)
@@ -1431,7 +1531,8 @@ checkFormatting <- function(pkgdir, nlines=6)
         handleMessage(
             "See http://bioconductor.org/developers/how-to/coding-style/")
         handleMessage(
-            "See FormatR package: https://cran.r-project.org/web/packages/formatR/index.html")
+            "See styler package: https://cran.r-project.org/package=styler ",
+            "as described in the BiocCheck vignette.")
     }
 }
 
@@ -1576,7 +1677,21 @@ checkDescription <- function(package_dir){
         return()
     })
     handleCheck("Checking for valid maintainer...")
-    if (("Authors@R" %in% colnames(dcf)) & any((c("Author","Maintainer") %in% colnames(dcf))))
-        handleWarning("Use Authors@R (preferred) or Author/Maintainer fields not both.")
+    if (("Authors@R" %in% colnames(dcf)) & any((c("Author","Maintainer") %in% colnames(dcf)))){
+        handleError("Use Authors@R field not Author/Maintainer fields. Do not use both.")
+    } else {
+        if (any((c("Author","Maintainer") %in% colnames(dcf))))
+            handleError("Do not use Author/Maintainer fields. Use Authors@R.")
+    }
+}
 
+checkForCitationFile <- function(package_dir) {
+    citfile_location <- file.path(package_dir, "inst", "CITATION")
+    if(file.exists(citfile_location)) {
+        handleCheck("Checking that provided CITATION file is correctly formatted...")
+        cit <- tryCatch(
+            readCitationFile(citfile_location),
+            error = function(e)
+                handleNote("CITATION file might be not correctly formatted"))
+    }
 }
